/*=========================================================================*/
/*                          Written by James Tanner                        */
/*                  https://www.facebook.com/TheMeroving1an                */
/*                                  IAmOrion                               */
/*=========================================================================*/

#include "Settings.h"
#include "Variables.h"
#include "Graphics.h"

/*=========================================================================*/
/*                             ADAFRUIT NEOPIXEL                           */
/*=========================================================================*/

#include <Adafruit_NeoPixel.h>
Adafruit_NeoPixel pixels = Adafruit_NeoPixel(NUMPIXELS, NEOPIXEL_PIN, NEO_GRB + NEO_KHZ800);

/*=========================================================================*/
/*                             U8G2 OLED LIBRARY                           */
/*=========================================================================*/

#include <U8g2lib.h>
U8G2_SSD1306_128X64_NONAME_F_HW_I2C u8g2(U8G2_R0, U8X8_PIN_NONE);

/*=========================================================================*/
/*                             RTC CLOCK LIBRARY                           */
/*=========================================================================*/

#include <Wire.h>
#include "RTClib.h"
RTC_DS3231 rtc;

/*=========================================================================*/
/*                                  WATCHDOG                               */
/*=========================================================================*/

#ifdef USE_WATCHDOG
  #include <Adafruit_SleepyDog.h>
#endif

void setup() {

  pixels.begin();
  pixels.setBrightness(25);
  for(int i=0;i<NUMPIXELS;i++){
    pixels.setPixelColor(i, pixels.Color(255,255,255));
    pixels.show();
  }

  #ifdef SERIAL_DEBUG
    Serial.begin(115200);
    #ifdef WAIT_FOR_SERIAL
      while(!Serial);
    #endif
    Serial.println(F("Begin..."));
  #endif 

  for(int i=0;i<NUMPIXELS;i++){
    pixels.setPixelColor(i, pixels.Color(255,0,255));
    pixels.show();
  }

  u8g2.begin();
  u8g2.clearBuffer();
  welcome();
  drawLogo();
  u8g2.sendBuffer();

  if (! rtc.begin()) {
    for(int i=0;i<NUMPIXELS;i++){
      pixels.setPixelColor(i, pixels.Color(0,255,0));
    }
    pixels.show();
    while (1);
  }
  if (rtc.lostPower()) {
    rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
    // This line sets the RTC with an explicit date & time, for example to set. E.g April 6, 2019 at 20:45 (8:45pm) you would call:
    // https://time.is/London
    // rtc.adjust(DateTime(2019, 6, 21, 20, 45, 0));
  }

  //rtc.adjust(DateTime(2019, 4, 8, 20, 06, 30));

  delay(SPLASH_SCREEN_DELAY);
  u8g2.clearDisplay();

  #ifdef USE_WATCHDOG
    Watchdog.enable(5000);
  #endif

  for (int i=0;i<4;i++) {
    checkJackResistance(i);
  }
  statusLEDs();

  targetTime = millis() + 1;
}

void loop() {

  for (int i=0;i<4;i++) {
    checkJackResistance(i);
  }

  statusLEDs();
  updateTime();
  updateScreen();
  checkScreenTimeout();

  #ifdef USE_WATCHDOG
    Watchdog.reset();
  #endif
}

void checkJackResistance(int socket) {
  float resistance = JACK_RESISTANCES[socket];
  int voltage = analogRead(Pins[socket]);
  float calc = voltage*VIN;
  VOUT = (calc)/1023.0;
  calc = VOUT/(VIN-VOUT);
  resistance = JACK_RESISTANCES[socket]*calc;

  if (resistance < R_MIN) {
    if (!INSERTED[socket]) {
      INSERTED[socket]=1;
    }
  } else {
    if (INSERTED[socket]) {
      INSERTED[socket]=0; 
    }
  }

  if (INSERTED[socket] != LAST_INSERTED[socket]) {
    LAST_INSERTED[socket] = INSERTED[socket];
    ACTIVE_SOCKET = socket;
    if (INSERTED[socket]) {
      #ifdef SERIAL_DEBUG
        Serial.print(F("PLUG IN SOCKET ")); Serial.print(socket+1); Serial.println(F(" INSERTED"));
      #endif
      if (P1_MIN < resistance && resistance < P1_MAX) {
        ACTIVE_JACK = 1;
      } else if (P2_MIN < resistance && resistance < P2_MAX) {
        ACTIVE_JACK = 2;
      } else if (P3_MIN < resistance && resistance < P3_MAX) {
        ACTIVE_JACK = 3;
      } else if (P4_MIN < resistance && resistance < P4_MAX) {
        ACTIVE_JACK = 4;
      //} else if (P5_MIN < resistance && resistance < P5_MAX) {
      } else {
        ACTIVE_JACK = 5;
      }
      SOCKET_1234_HAS_PLUGTYPE_X[socket] = ACTIVE_JACK;
    } else if (!INSERTED[socket]) {
      #ifdef SERIAL_DEBUG
        Serial.print(F("PLUG IN SOCKET ")); Serial.print(socket+1); Serial.println(F(" REMOVED"));
      #endif
      ACTIVE_JACK = SOCKET_1234_HAS_PLUGTYPE_X[socket];
    }

    #ifdef SERIAL_DEBUG
      Serial.print(F("ACTIVE JACK = ")); Serial.println(ACTIVE_JACK);
      int len = sizeof(SOCKET_1234_HAS_PLUGTYPE_X)/sizeof(SOCKET_1234_HAS_PLUGTYPE_X[0]);
      for (int i=0;i<len;i++) {
        Serial.print(F("SOCKET_1234_HAS_PLUGTYPE_X[")); Serial.print(i); Serial.print(F("] = "));
        Serial.println(SOCKET_1234_HAS_PLUGTYPE_X[i]);
      }
      Serial.println();
      if (INSERTED[socket]) {
        Serial.print(F("Plug in socket ")); Serial.print(socket+1); Serial.print(F(" has a resitance of: ")); Serial.println(resistance);
      }
    #endif
    
    ACTIVE=true; previousMillis = millis();
  }
}

void statusLEDs(void) {
  for(int i=0;i<NUMPIXELS;i++){
    if (INSERTED[i]) {
      pixels.setPixelColor(i, pixels.Color(0,0,255));
    } else {
      pixels.setPixelColor(i, pixels.Color(255,0,0));
    }
    pixels.show();
  }
}

void updateTime(void) {
  DateTime now = rtc.now();
  
  if (targetTime < millis()) {
    targetTime = millis()+1000;
    COLON_VISIBLE = !COLON_VISIBLE;
    
    hh = now.hour(),DEC;
    mm = now.minute(),DEC;
    ss = now.second(),DEC;

    if (hh < 10) { str_hh = "0" + String(hh); } else { str_hh = String(hh); }
    if (mm < 10) { str_mm = "0" + String(mm); } else { str_mm = String(mm); }
    if (ss < 10) { str_ss = "0" + String(ss); } else { str_ss = String(ss); }
    
    if (COLON_VISIBLE) {
      strTime = str_hh + ":" + str_mm;
    } else {
      strTime = str_hh + " " + str_mm;
    }
  }
}

void updateScreen() {
  char charString[20];
  
  u8g2.clearBuffer();
  u8g2.setFont(u8g2_font_ncenR14_tr);
  if (ACTIVE) {
    if (INSERTED[ACTIVE_SOCKET]) {
      stringBuilder = "+ " + String(PLUGS[ACTIVE_JACK-1]) + " in " + (ACTIVE_SOCKET+1);
    } else if (!(INSERTED[ACTIVE_SOCKET])) {
      stringBuilder = "- " + String(PLUGS[ACTIVE_JACK-1]) + " in " + (ACTIVE_SOCKET+1);
    }
    switch(ACTIVE_JACK) {
      case 1: drawZ4(); break; 
      case 2: break; 
      case 3: break; 
      case 4: break;
      case 5: drawStandardPlug(); break;
      default: break;
    }
  } else {
    clearImage();
    stringBuilder = strTime;
  }
  stringBuilder.toCharArray(charString,20);
  u8g2.drawStr(((128-(u8g2.getStrWidth(charString)))/2),15,charString);
  u8g2.sendBuffer();
}

void welcome(void) {
  u8g2.setFont(u8g2_font_profont22_mr);
  u8g2.drawStr(((128-(u8g2.getStrWidth("IAmOrion")))/2),15,"IAmOrion");      // Center Aligned
}

void drawLogo(void) {
  u8g2.drawXBMP(40,16,IAmOrion_width,IAmOrion_height,IAmOrion);
}

void drawZ4(void) {
  u8g2.drawXBMP(0,28,Z4_width,Z4_height,Z4);
}

void drawStandardPlug(void) {
  u8g2.drawXBMP(0,16,standardplug_width,standardplug_height,standardplug);
}

void clearImage(void) {
  u8g2.drawXBMP(0,16,clear_width,clear_height,clear_bits);
}

void checkScreenTimeout(void) {
  unsigned long currentMillis = millis();
  if (ACTIVE) {
    if (currentMillis - previousMillis >= DISPLAY_TIMOUT) {
      previousMillis = currentMillis;
      DoReset();
    }
  } else {
    previousMillis = currentMillis;
  }
}

void DoReset(void) {
  ACTIVE=false; ACTIVE_JACK=0; ACTIVE_SOCKET=0;
}
